@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -59,11 +59,16 @@ PrintPreview.isRunning = function() {
 /**
  * Starts the print preview.
  */
-PrintPreview.start = function(initialAction) {
+PrintPreview.start = function(initialAction, instance) {
     var di = EAction.getDocumentInterface();
     var a = di.getDefaultAction();
     a.finishEvent();
-    PrintPreview.instance = new PrintPreviewImpl();
+    if (isNull(instance)) {
+        PrintPreview.instance = new PrintPreviewImpl();
+    }
+    else {
+        PrintPreview.instance = instance;
+    }
     if (!isNull(initialAction)) {
         PrintPreview.instance.initialAction = initialAction;
     }
@@ -101,7 +106,6 @@ PrintPreview.prototype.beginEvent = function() {
     }
 
     this.terminate();
-    return;
 };
 
 
@@ -134,9 +138,13 @@ function PrintPreviewImpl(guiAction) {
     this.saveView = false;
     this.savedScale = undefined;
     this.savedOffset = undefined;
+    this.savedColumns = undefined;
+    this.savedRows = undefined;
+
+    this.optOutRelativeZeroResume = true;
 }
 
-PrintPreviewImpl.prototype = NewFile.getDefaultAction();
+PrintPreviewImpl.prototype = NewFile.getDefaultAction(false);
 PrintPreviewImpl.includeBasePath = includeBasePath;
 
 PrintPreviewImpl.State = {
@@ -146,9 +154,15 @@ PrintPreviewImpl.State = {
 PrintPreviewImpl.prototype.beginEvent = function() {
     var di = this.getDocumentInterface();
 
-    if (this.saveView===true) {
+    var appWin = RMainWindowQt.getMainWindow();
+    var initialZoom = appWin.property("PrintPreview/InitialZoom");
+
+    if (initialZoom==="View") {
         this.savedScale = Print.getScale(this.getDocument());
         this.savedOffset = Print.getOffset(this.getDocument());
+        this.savedColumns = Print.getColumns(this.getDocument());
+        this.savedRows = Print.getRows(this.getDocument());
+        this.saveView = true;
     }
 
 //    if (!isNull(this.guiAction)) {
@@ -186,12 +200,12 @@ PrintPreviewImpl.prototype.beginEvent = function() {
     var action = RGuiAction.getByScriptFile("scripts/Edit/DrawingPreferences/DrawingPreferences.js");
     action.triggered.connect(this, "updateBackgroundDecoration");
 
-    var appWin = RMainWindowQt.getMainWindow();
-    var initialZoom = appWin.property("PrintPreview/InitialZoom");
     if (initialZoom==="Auto") {
         this.slotAutoFitDrawing();
     }
     else if (initialZoom==="View") {
+        Print.setColumns(di, 1);
+        Print.setRows(di, 1);
         this.slotAutoFitBox(this.view.getBox());
     }
 
@@ -222,7 +236,7 @@ PrintPreviewImpl.prototype.setState = function(state) {
     this.parentClass.prototype.setState.call(this, state);
 
     if (this.state === PrintPreviewImpl.State.SettingOffset) {
-        EAction.getDocumentInterface().setClickMode(RAction.PickingDisabled);
+        this.getDocumentInterface().setClickMode(RAction.PickingDisabled);
         this.setCursor(this.cursor, "PrintPreviewOffsetCursor");
         EAction.showMainTools();
         EAction.getMainWindow().setLeftMouseTip(qsTr("Drag to move paper"));
@@ -241,10 +255,12 @@ PrintPreviewImpl.prototype.suspendEvent = function() {
 };
 
 PrintPreviewImpl.prototype.finishEvent = function() {
-    qDebug("PrintPreviewImpl.prototype.finishEvent");
     this.parentClass.prototype.finishEvent.call(this);
     var di = this.getDocumentInterface();
 
+    var appWin = RMainWindowQt.getMainWindow();
+    var initialZoom = appWin.property("PrintPreview/InitialZoom");
+
     if (!isNull(this.view)) {
         this.view.setPrintPreview(false);
         this.view.setBackgroundColor(this.bgColor);
@@ -273,14 +289,19 @@ PrintPreviewImpl.prototype.finishEvent = function() {
 
     if (this.saveView===true) {
         if (!isNull(this.savedScale)) {
-            Print.setScale(di.getDocument(), this.savedScale);
+            Print.setScale(di, this.savedScale);
         }
         if (!isNull(this.savedOffset)) {
-            Print.setOffset(di.getDocument(), this.savedOffset);
+            Print.setOffset(di, this.savedOffset);
+        }
+        if (!isNull(this.savedColumns)) {
+            Print.setColumns(di, this.savedColumns);
+        }
+        if (!isNull(this.savedRows)) {
+            Print.setRows(di, this.savedRows);
         }
     }
 
-    var appWin = RMainWindowQt.getMainWindow();
     if (!isNull(this.pAdapter)) {
         appWin.removePreferencesListener(this.pAdapter);
     }
@@ -346,13 +367,14 @@ PrintPreviewImpl.prototype.mouseMoveEvent = function(event) {
         }
         this.panOffsetOrigin = panTarget;
 
-        var document = this.getDocument();
+        var di = this.getDocumentInterface();
+        var document = di.getDocument();
         var offset = Print.getOffset(document);
         offset.x += this.view.mapDistanceFromView(panDelta.x);
         offset.y -= this.view.mapDistanceFromView(panDelta.y);
 
         // update document variables
-        Print.setOffset(document, offset);
+        Print.setOffset(di, offset);
 
         this.updateBackgroundTransform();
 
@@ -414,9 +436,11 @@ PrintPreviewImpl.prototype.showUiOptions = function(resume) {
     }
 
     if (Print.getHairlineMode(document)) {
-        widgets["Hairline"].blockSignals(true);
-        widgets["Hairline"].checked=true;
-        widgets["Hairline"].blockSignals(false);
+        if (!isNull(widgets["Hairline"])) {
+            widgets["Hairline"].blockSignals(true);
+            widgets["Hairline"].checked=true;
+            widgets["Hairline"].blockSignals(false);
+        }
     }
 
     widgets["Portrait"].blockSignals(true);
@@ -483,11 +507,26 @@ PrintPreviewImpl.prototype.updateBackgroundDecoration = function() {
         return;
     }
 
+    var colBg;
+    var colShadow;
+    var colBorder;
+
+    if (RSettings.hasDarkGuiBackground()) {
+        colBg = "#888888";
+        colShadow = "#333333";
+        colBorder = "black";
+    }
+    else {
+        colBg = "lightgray";
+        colShadow = "gray";
+        colBorder = "#c8c8c8";
+    }
+
     path = new RPainterPath();
     path.setPen(new QPen(Qt.NoPen));
-    path.setBrush(new QBrush(new QColor("lightgray")));
+    path.setBrush(new QBrush(new QColor(colBg)));
     path.addRect(new QRectF(-1.0e8, -1.0e8, 2.0e8, 2.0e8));
-    this.view.addToBackground(path);
+    this.view.addToBackground(RGraphicsSceneDrawable.createFromPainterPath(path));
 
     // page border with shadow
     if (Print.getShowPaperBorders(document)) {
@@ -495,9 +534,9 @@ PrintPreviewImpl.prototype.updateBackgroundDecoration = function() {
         for (i = 0; i < pages.length; ++i) {
             path = new RPainterPath();
             path.setPen(new QPen(Qt.NoPen));
-            path.setBrush(new QBrush(new QColor("gray")));
+            path.setBrush(new QBrush(new QColor(colShadow)));
             this.drawShadow(path, pages[i]);
-            this.view.addToBackground(path);
+            this.view.addToBackground(RGraphicsSceneDrawable.createFromPainterPath(path));
         }
 
         // paper background
@@ -508,16 +547,16 @@ PrintPreviewImpl.prototype.updateBackgroundDecoration = function() {
                     backgroundColor.blue());
             path.setBrush(new QBrush(color));
             this.drawPaper(path, pages[i]);
-            this.view.addToBackground(path);
+            this.view.addToBackground(RGraphicsSceneDrawable.createFromPainterPath(path));
         }
 
         // paper border
         for (i = 0; i < pages.length; ++i) {
             path = new RPainterPath();
-            path.setPen(new QPen(new QColor(0xc8, 0xc8, 0xc8)));
+            path.setPen(new QPen(new QColor(colBorder)));
             path.setBrush(new QBrush(Qt.NoBrush));
             this.drawPaper(path, pages[i]);
-            this.view.addToBackground(path);
+            this.view.addToBackground(RGraphicsSceneDrawable.createFromPainterPath(path));
         }
     }
 
@@ -539,17 +578,17 @@ PrintPreviewImpl.prototype.updateBackgroundDecoration = function() {
     for (i = 0; i < pages.length; ++i) {
         this.drawGlueMargins(path, pages[i]);
     }
-    this.view.addToBackground(path);
+    this.view.addToBackground(RGraphicsSceneDrawable.createFromPainterPath(path));
 
-    // TODO: hook for page tags
+    // hook for page tags, footer, etc.:
     this.addDecorations(pages);
 
-    // crop marks
+    // crop marks:
     if (Print.getPrintCropMarks(document)) {
         for ( i = 0; i < pages.length; ++i) {
             path = new RPainterPath();
             Print.drawCropMarks(document, path, pages[i], false);
-            this.view.addToBackground(path);
+            this.view.addToBackground(RGraphicsSceneDrawable.createFromPainterPath(path));
         }
     }
 
@@ -565,7 +604,7 @@ PrintPreviewImpl.prototype.addDecorations = function(pages) {};
  * Updates the background decoration transformation.
  */
 PrintPreviewImpl.prototype.updateBackgroundTransform = function() {
-    var document = EAction.getDocument();
+    var document = this.getDocument();
 
     if (isNull(document)) {
         return;
@@ -588,7 +627,7 @@ PrintPreviewImpl.prototype.updateBackgroundTransform = function() {
  * Draws the paper border for the given page.
  */
 PrintPreviewImpl.prototype.drawPaper = function(path, border) {
-    var document = EAction.getDocument();
+    var document = this.getDocument();
     var paperBorder = Print.getPaperBorder(document, border);
     if (!paperBorder.isValid()) {
         return;
@@ -610,7 +649,7 @@ PrintPreviewImpl.prototype.drawGlueMargins = function(path, border) {
  * Draws the shadow for the given page.
  */
 PrintPreviewImpl.prototype.drawShadow = function(path, border) {
-    var document = EAction.getDocument();
+    var document = this.getDocument();
     var paper = Print.getPaperBorder(document, border);
     var offset = Print.getPaperSizeMM(document).width() * 0.02;
     var shadow = paper.adjusted(offset, -offset, offset, -offset);
@@ -622,27 +661,36 @@ PrintPreviewImpl.prototype.drawShadow = function(path, border) {
 
 PrintPreviewImpl.prototype.slotPdfExport = function() {
     var appWin = EAction.getMainWindow();
-    var fileName = EAction.getDocument().getFileName();
+    var fileName = this.getDocument().getFileName();
     var initialFileName = QDir.homePath();
     if (fileName!=="") {
         var fileInfo = new QFileInfo(fileName);
         initialFileName = fileInfo.absoluteFilePath();
     }
 
-    var filters = [ "PDF File (*.pdf)" ];
+    var filterStrings = [ "PDF File (*.pdf)",  "PDF/A-1B File (*.pdf)"];
+    if (RSettings.getQtVersion()<0x050A00) {
+        filterStrings = [ "PDF File (*.pdf)" ];
+    }
+
+    filterStrings = translateFilterStrings(filterStrings);
 
     var ret = File.getSaveFileName(appWin, qsTr("Export to PDF"),
-                   initialFileName, filters);
+                   initialFileName, filterStrings);
 
     if (isNull(ret)) {
         return;
     }
 
     var pdfFile = ret[0];
+    var pdfVersion = undefined;
+    if (ret[1].indexOf("PDF/A")!==-1) {
+        pdfVersion = "A-1B";
+    }
 
     appWin.handleUserMessage(qsTr("Exporting to %1...").arg(pdfFile));
 
-    var success = this.slotPrint(pdfFile);
+    var success = this.slotPrint(pdfFile, pdfVersion);
 
     if (success) {
         appWin.handleUserMessage(qsTr("Export complete: %1").arg(pdfFile));
@@ -660,18 +708,18 @@ PrintPreviewImpl.slotPdfExport = function() {
     pp.slotPdfExport();
 };
 
-PrintPreviewImpl.prototype.slotPrint = function(pdfFile) {
-    return PrintPreviewImpl.slotPrint(pdfFile);
+PrintPreviewImpl.prototype.slotPrint = function(pdfFile, pdfVersion) {
+    return PrintPreviewImpl.slotPrint(pdfFile, pdfVersion);
 };
 
 /**
  * Prints the drawing or exports it to the given PDF file.
  */
-PrintPreviewImpl.slotPrint = function(pdfFile) {
+PrintPreviewImpl.slotPrint = function(pdfFile, pdfVersion) {
     var mdiChild = EAction.getMdiChild();
     var view = mdiChild.getLastKnownViewWithFocus();
     var print = new Print(undefined, EAction.getDocument(), view);
-    return print.print(pdfFile);
+    return print.print(pdfFile, undefined, pdfVersion);
 };
 
 /**
@@ -682,27 +730,25 @@ PrintPreviewImpl.prototype.slotScaleChanged = function(scaleString) {
         return;
     }
 
-    var document = this.getDocument();
-
-    if (isNull(document) || this.updateDisabled===true) {
+    var di = this.getDocumentInterface();
+    if (isNull(di) || this.updateDisabled===true) {
         return;
     }
 
-    Print.setScaleString(document, scaleString);
+    Print.setScaleString(di, scaleString);
 
     this.updateBackgroundTransform();
 
-    // update pattern scale accordint to drawing scale:
-    var di = EAction.getDocumentInterface();
+    // update pattern scale according to drawing scale:
     di.regenerateScenes();
 };
 
-PrintPreviewImpl.setScale = function(document, scale) {
-    PrintPreviewImpl.setScaleString(document, sprintf("%.6f", scale));
+PrintPreviewImpl.setScale = function(di, scale) {
+    PrintPreviewImpl.setScaleString(di, sprintf("%.6f", scale));
 };
 
-PrintPreviewImpl.setScaleString = function(document, scaleString) {
-    Print.setScaleString(document, scaleString);
+PrintPreviewImpl.setScaleString = function(di, scaleString) {
+    Print.setScaleString(di, scaleString);
 
     PrintPreviewImpl.updateScaleString(document);
 };
@@ -751,7 +797,7 @@ PrintPreviewImpl.prototype.slotFullColorChanged = function(checked) {
         return;
     }
     if (checked) {
-        Print.setColorMode(EAction.getDocument(), RGraphicsView.FullColor);
+        Print.setColorMode(this.getDocumentInterface(), RGraphicsView.FullColor);
         this.colorModeUpdate();
     }
 };
@@ -761,7 +807,7 @@ PrintPreviewImpl.prototype.slotBlackWhiteChanged = function(checked) {
         return;
     }
     if (checked) {
-        Print.setColorMode(EAction.getDocument(), RGraphicsView.BlackWhite);
+        Print.setColorMode(this.getDocumentInterface(), RGraphicsView.BlackWhite);
         this.colorModeUpdate();
     }
 };
@@ -771,14 +817,14 @@ PrintPreviewImpl.prototype.slotGrayscaleChanged = function(checked) {
         return;
     }
     if (checked) {
-        Print.setColorMode(EAction.getDocument(), RGraphicsView.GrayScale);
+        Print.setColorMode(this.getDocumentInterface(), RGraphicsView.GrayScale);
         this.colorModeUpdate();
     }
 };
 
 PrintPreviewImpl.prototype.colorModeUpdate = function() {
     if (!isNull(this.view)) {
-        this.view.setColorMode(Print.getColorMode(EAction.getDocument()));
+        this.view.setColorMode(Print.getColorMode(this.getDocument()));
         this.view.regenerate(true);
     }    
 };
@@ -787,7 +833,7 @@ PrintPreviewImpl.prototype.slotHairlineChanged = function(checked) {
     if (this.updateDisabled===true) {
         return;
     }
-    Print.setHairlineMode(EAction.getDocument(), checked);
+    Print.setHairlineMode(this.getDocumentInterface(), checked);
     if (!isNull(this.view)) {
         this.view.setHairlineMode(checked)
         this.view.regenerate(true);
@@ -802,8 +848,8 @@ PrintPreviewImpl.prototype.slotShowPaperBordersChanged = function(checked) {
     if (this.updateDisabled===true) {
         return;
     }
-    var document = EAction.getDocument();
-    Print.setShowPaperBorders(document, checked);
+    var di = this.getDocumentInterface();
+    Print.setShowPaperBorders(di, checked);
 
     this.updateBackgroundDecoration();
 };
@@ -815,8 +861,8 @@ PrintPreviewImpl.prototype.slotPrintCropMarksChanged = function(checked) {
     if (this.updateDisabled===true) {
         return;
     }
-    var document = EAction.getDocument();
-    Print.setPrintCropMarks(document, checked);
+    var di = this.getDocumentInterface();
+    Print.setPrintCropMarks(di, checked);
 
     this.updateBackgroundDecoration();
 };
@@ -825,12 +871,13 @@ PrintPreviewImpl.prototype.slotPrintCropMarksChanged = function(checked) {
  * Auto fit box to page.
  */
 PrintPreviewImpl.prototype.slotAutoFitBox = function(box) {
-    var document = this.getDocument();
-    Print.autoFitBox(document, box);
-    PrintPreviewImpl.updateScaleString(document);
+    var di = this.getDocumentInterface();
+    var doc = di.getDocument();
+    Print.autoFitBox(di, box);
+    PrintPreviewImpl.updateScaleString(doc);
 
     // needed to update pattern scaling according to drawing scale:
-    var di = this.getDocumentInterface();
+    //var di = this.getDocumentInterface();
     di.regenerateScenes();
 
     this.updateBackgroundTransform();
@@ -843,12 +890,12 @@ PrintPreviewImpl.prototype.slotAutoFitBox = function(box) {
  * Auto fit drawing button clicked in options toolbar.
  */
 PrintPreviewImpl.prototype.slotAutoFitDrawing = function() {
-    var document = this.getDocument();
-    Print.autoFitDrawing(document);
+    var di = this.getDocumentInterface();
+    var document = di.getDocument();
+    Print.autoFitDrawing(di);
     PrintPreviewImpl.updateScaleString(document);
 
     // needed to update pattern scaling according to drawing scale:
-    var di = this.getDocumentInterface();
     di.regenerateScenes();
 
     this.updateBackgroundTransform();
@@ -858,7 +905,7 @@ PrintPreviewImpl.prototype.slotAutoFitDrawing = function() {
 };
 
 PrintPreviewImpl.prototype.slotAutoCenter = function() {
-    Print.autoCenter(this.getDocument());
+    Print.autoCenter(this.getDocumentInterface());
 
     this.updateBackgroundTransform();
     this.slotAutoZoomToPage();
@@ -869,8 +916,8 @@ PrintPreviewImpl.prototype.slotPortraitChanged = function() {
     if (this.updateDisabled===true) {
         return;
     }
-    var document = this.getDocument();
-    Print.setPageOrientationEnum(document, QPrinter.Portrait);
+    var di = this.getDocumentInterface();
+    Print.setPageOrientationEnum(di, QPrinter.Portrait);
 
     this.updateBackgroundDecoration();
     this.slotAutoZoomToPage();
@@ -880,8 +927,8 @@ PrintPreviewImpl.prototype.slotLandscapeChanged = function() {
     if (this.updateDisabled===true) {
         return;
     }
-    var document = this.getDocument();
-    Print.setPageOrientationEnum(document, QPrinter.Landscape);
+    var di = this.getDocumentInterface();
+    Print.setPageOrientationEnum(di, QPrinter.Landscape);
 
     this.updateBackgroundDecoration();
     this.slotAutoZoomToPage();
